*************************************************************************************
*														*
*	.BMP file loader											*
*														*
*	This version only supports the displaying of 256 colour bitmaps			*
*														*
*************************************************************************************

xyoff	EQU		100				* x,y offset from screen edge
xstep	EQU		40				* block x step
ystep	EQU		36				* block y step
xwide	EQU		31				* block x width
ywide	EQU		23				* block y width

* this is the offset from the top left corner of the image to the top left corner of
* the top left button

x_tl	EQU		10				* x offset of top left button in image
y_tl	EQU		38				* y offset of top left button in image

* this is the name of the 256 colour bitmap file to use as the background
name
	dc.b	'lightsout.bmp',0			* background file name
*##	dc.b	'simpsons.bmp',0			* alternate background file name
	ds.w	0					* ensure even


*************************************************************************************
*
* opens and loads an image

abort_load
	MOVEQ		#50,d0			* else close all files
	TRAP		#15

abort_display
	RTS

load_image
	LEA		name(pc),a1			* set the file name pointer
	MOVE		#51,d0			* open an existing file
	TRAP		#15

	TST.w		d0				* did the file open
	BNE.s		abort_load			* if not then abort the file load

	LEA		im_file(a3),a1		* point to the file buffer
	MOVE.l	#$FF0000,d2			* set the byte count to load everything
	MOVEQ		#53,d0			* read from the file
	TRAP		#15

	MOVEQ		#56,d0			* close the file
	TRAP		#15

	MOVE.w	(a1)+,d0			* get the signature word
	CMPI.w	#'BM',d0			* compare the signature word with "BM"
	BNE.s		abort_display		* if incorrect abort the file display

	LEA		$0C(a1),a1			* skip the rest of the file header

	MOVEQ		#$28,d2			* set the size that is expected
	BSR		get_long			* get the image header size
	CMP.l		d2,d0				* compare them
	BNE.s		abort_display		* if incorrect abort the file display

	BSR		get_long			* get the image width
	MOVE.l	d0,im_width(a3)		* save the image width
	BSR		get_long			* get the image height
	MOVE.l	d0,im_height(a3)		* save the image height

	BSR.s		get_word			* get the number of planes
	CMPI.w	#1,d0				* compare with expected number
	BNE.s		abort_display		* if incorrect abort the file display

	BSR.s		get_word			* get the image depth
	CMPI.w	#8,d0				* compare the image depth with 256 colour
	BNE.s		abort_display		* if incorrect abort the file display

	MOVE.w	d0,im_depth(a3)		* save the image depth

	BSR.s		get_long			* get the compression type
	TST.l		d0				* test for uncompressed file
	BNE.s		abort_display		* if incorrect abort the file display

	LEA		$14(a1),a1			* skip the rest of the header

	BSR.s		load_palette		* load the palette

	MOVE.l	a1,-(sp)			* save the file pointer as the image needs to
							* be rendered in both screem buffer pages
	BSR.s		display_image		* render the image to the screen

	MOVEQ		#94,d0			* copy screen buffer to main
	TRAP		#15

	MOVEA.l	(sp)+,a1			* restore the file pointer and ...
*##	BRA.s		display_image		* render the image to the screen


*************************************************************************************
*
* display the image

display_image
	MOVE.w	#$FF00,d1			* clear screen
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	MOVE.l	im_height(a3),d7		* get the image height
	SUBQ.l	#1,d7				* adjust for the loop type
	MOVE.w	im_y(a3),d5			* get the image y origin
	ADD.w		d7,d5				* set the end y co-ordinate
bbp8_y_loop
	MOVE.l	im_width(a3),d6		* get the image width
	MOVEQ		#1,d3				* set odd byte flag
	AND.l		d6,d3				* mask line length
	SUBQ.l	#1,d6				* adjust for the loop type
	MOVE.w	im_x(a3),d4			* get the image x origin
bbp8_x_loop
	MOVEQ		#0,d0				* clear the longword
	MOVE.b	(a1)+,d0			* get the pixel byte
	ASL.w		#2,d0				* 4 bytes per longword colour
	MOVE.l	im_palette(a3,d0.w),d1	* get the colour

	MOVEQ		#80,d0			* set the pen colour
	TRAP		#15

	MOVE.w	d4,d1				* copy the x co-ordinate
	MOVE.w	d5,d2				* copy the y co-ordinate

	MOVEQ		#82,d0			* draw a pixel
	TRAP		#15

	ADDQ.w	#1,d4				* increment the x co-ordinate
	DBF		d6,bbp8_x_loop		* loop for next pixel

	TST.l		d3				* test the odd byte flag
	BEQ.s		no_odd_byte			* skip read if no odd byte

	MOVE.b	(a1)+,d0			* else get the odd byte
no_odd_byte
	SUBQ.w	#1,d5				* decrement the y co-ordinate
	DBF		d7,bbp8_y_loop		* loop for next line

	RTS


*************************************************************************************
*
* get a little endian word, this could be at an even or an odd address

get_word
	MOVE.b	(a1)+,d0			* get the low byte
	ROL.w		#8,d0				* move it
	MOVE.b	(a1)+,d0			* get the high byte
	ROL.w		#8,d0				* swap the bytes
	RTS


*************************************************************************************
*
* get a little endian longword, this could be at an even or an odd address

get_long
	BSR.s		get_word			* get a little endian word
	SWAP		d0				* move it
	BSR.s		get_word			* get a little endian word
	SWAP		d0				* swap the words
	RTS


*************************************************************************************
*
* load the palette. this routine assumes a 256 colour palette.

load_palette
	LEA		im_palette(a3),a2		* point to the palette
	MOVE.w	#$00FF,d7			* set the number of colours - 1

* get the bitmap palette colour values. palette colour values are 32 bit values stored
* as B,G,R,$00. EASy68K wants 32 bit colour values in the form $00,B,G,R

load_palette_loop
	MOVEQ		#0,d0				* clear the longword
	MOVE.b	(a1)+,d0			* get the blue byte
	SWAP		d0				* move it to the high word
	MOVE.b	(a1)+,d0			* get the green byte
	ROL.w		#8,d0				* move it to the high byte
	MOVE.b	(a1)+,d0			* get the red byte
	ADDQ.w	#1,a1				* skip the zero byte

	MOVE.l	d0,(a2)+			* save the value to the palette
	DBF		d7,load_palette_loop	* loop for the next colour

	RTS


*************************************************************************************