*************************************************************************************
*														*
*	A recreation of the 'Lights Out' game by Tiger for the EASy68K simulator	*
*	V1.00													*
*														*
*	The aim of the game is to extinguish all the lights in the five by five		*
*	grid of lights. Clicking on a light with the mouse will toggle the state	*
*	of that light and all the vertically and horizontally adjacent lights.		*
*	Once the puzzle is comlete play advances to the next puzzle.			*
*														*
*	This game requires EASy68K 3.6 or later.							*
*														*
*	See destructions.txt for a more complete description.					*
*														*
*	The background is a 256 colour bitmap image loaded from the file			*
*	lightsout.bmp											*
*														*
*														*
*	More 68000 and other projects can be found on my website at ..			*
*														*
*	 http://www.themotionstore.com/leeedavison/index.html					*
*														*
*	.. or ..												*
*														*
*	 http://members.lycos.co.uk/leeedavison/index.html					*
*														*
*	mail : leeedavison@lycos.co.uk								*
*														*
*************************************************************************************

*************************************************************************************
*

	ORG	$1000

start
	BSR		initialise			* go setup game stuff
	BSR		load_image			* go load the game background image

* main loop. everything is done in this loop, the overall speed of the loop is set by
* waiting for the remainder of a fixed time to pass at the end of the loop

loop
	MOVEQ		#8,d0				* get time in 1/100 ths seconds
	TRAP		#15

	MOVE.l	d1,-(sp)			* push the time on the stack

	BSR		mouse_move			* handle mouse movement

	MOVE.w	game_mode(a3),d7		* get the game mode
	LEA		mode_table(pc),a1		* point to the mode branch table
	MOVE.w	(a1,d7.w),d7		* ge the branch offset
	JSR		(a1,d7.w)			* go do the mode routine

	BSR		do_draw			* draw the board
	BSR		gen_prng			* generate a new random number

* now wait for a while. this limits the loop to ten redraws a second without which
* this could run way to fast if the machine was capable

main_end
	MOVE.l	(sp)+,d7			* restore time in 1/100 ths from stack
wait_100ms
	MOVEQ		#8,d0				* get time in 1/100 ths seconds
	TRAP		#15

	SUB.l		d7,d1				* subtract previous time from current time
	CMP.b		#$09,d1			* compare with 9/100ths
	BMI.s		wait_100ms			* loop if time not up yet

* if you want to see how fast it can go comment out the BMI.s above

	BRA		loop				* loop forever


*------------------------------------------------------------------------------------
*
* the mode table, redirects execution depending on the game mode

mode_table
mode_welcome				EQU	*-mode_table
	dc.w	welcome_mode-mode_table		* welcome mode, chases the lights
mode_ready					EQU	*-mode_table
	dc.w	ready_mode-mode_table		* ready mode, displays a "+"
mode_wait					EQU	*-mode_table
	dc.w	wait_mode-mode_table		* wait mode, idles waiting for change
mode_start_play				EQU	*-mode_table
	dc.w	start_play_mode-mode_table	* start play mode, flash the level
mode_play					EQU	*-mode_table
	dc.w	play_mode-mode_table		* play mode, playing a game
mode_win					EQU	*-mode_table
	dc.w	win_mode-mode_table		* win display mode, chase, flash or X
mode_select					EQU	*-mode_table
	dc.w	selct_mode-mode_table		* select mode, mode 1 or mode 2
mode_help_hint				EQU	*-mode_table
	dc.w	help_hint_mode-mode_table	* help hint mode, flash the help hint
mode_save					EQU	*-mode_table
	dc.w	save_mode-mode_table		* save mode


*************************************************************************************
*
* does a lights chase around the game board as a welcome then drops into the ready
* mode

welcome_mode
	TST.w		chase_seq(a3)		* test if the sequence has started
	BPL.s		no_play_intro		* if so skip the intro

	BSR		play_intro			* play the intro noise
no_play_intro
	BSR		lights_chase		* do the lights chase
	TST.l		d1				* test if the sequence is complete
	BEQ.s		exit_welcome_mode		* if not just exit

	MOVE.w	#mode_ready,game_mode(a3)
							* else set the game mode to ready mode
exit_welcome_mode
	RTS


*************************************************************************************
*
* puts a "+" on the display and goes to the wait mode

ready_mode
	MOVE.l	#0,button_l(a3)		* clear any mouse button presses
	LEA		save_flash(pc),a0		* point to "+" pattern
	MOVE.l	(a0),display_board(a3)	* set the board pattern to "+"
	MOVE.w	#-1,draw_board(a3)	* set the draw board flag
	MOVE.w	#mode_wait,game_mode(a3)
							* set the game mode to wait mode
	MOVE.w	#mode_wait,last_mode(a3)
							* set the last mode to wait mode
	RTS


*************************************************************************************
*
* waits for the user to do something

wait_mode
	BSR		control_buttons		* handle the control buttons
	MOVE.w	#0,button_l(a3)		* clear the left button flag
	RTS


*************************************************************************************
*
* does the flash the game level then drops into play mode. if the last_board variable
* is set that game is restarted, otherwise a new game is started.

start_play_mode
	BSR		start_flash			* do the start of the game sequence
	TST.l		d1				* test if the sequence is complete
	BEQ.s		exit_start_play_mode	* if not just exit

* the start game sequence has finished so start a game

start_game
	MOVE.l	last_board(a3),d1		* get the last game board
	BNE.s		do_this_game		* if game present go do it

* there is no previous game so start a new game

	MOVEQ		#0,d0				* clear the longword for a possible divide
	MOVE.w	play_level(a3),d0		* get the play level

	LEA		level_help(pc),a0		* point to the level help count table
	MOVE.w	d0,d1				* copy the play level
	ADD.w		d1,d1				* two bytes per level help count
	MOVE.w	(a0,d1.w),help_count(a3)
							* set the help count for this level

	LEA		fifty_puzzle(pc),a0	* point to thelevel game puzzle table
	CMP.w		#1,select_mode(a3)	* compare the select mode with level game
	BEQ.s		do_start			* if we are playing the level game go do it

* else the select mode is random game, so select a new random puzzle

	LEA		random_puzzle(pc),a0	* point to the random puzzle table
	BSR		get_moves			* get the minimum number of moves - 6
	MULU.w	#400,d0			* multiply by each level table length
	LEA		(a0,d0.w),a0		* point to this level's table
	MOVEQ		#100,d0			* want a number from 0 to 99
	BSR		get_prng			* get a random number
do_start
	ASL.w		#2,d0				* * 4 bytes per board pattern
	MOVE.l	(a0,d0.w),d1		* get the initial board pattern
	MOVE.l	d1,last_board(a3)		* save this as the last game board
do_this_game
	MOVE.l	d1,game_board(a3)		* initialise the board
	MOVE.w	#0,moves_made(a3)		* clear the moves made count
	MOVE.w	#0,button_l(a3)		* clear the left button flag
	MOVE.l	#0,solution(a3)		* clear the solution longword
	MOVE.w	#mode_play,game_mode(a3)
							* set the game mode to play mode
exit_start_play_mode
	RTS


*************************************************************************************
*
* play the game and possibly set for a win

play_mode
	BSR		control_buttons		* handle the control buttons

	CMPI.w	#2,select_mode(a3)	* compare the select mode with random play
	BEQ.s		do_play_mode		* if random play always go play the game

	CMPI.w	#50,play_level(a3)	* compare the play level with done
	BEQ.s		skip_win			* if already level 51 skip playing

do_play_mode
	BSR		light_buttons		* handle the light buttons

	TST.l		game_board(a3)		* test the board state
	BNE.s		skip_win			* if not won skip updating the level

	MOVEQ		#0,d0				* clear the longword for a possible divide
	MOVE.w	play_level(a3),d0		* get the play level

	BSR		get_moves			* get the minimum number of moves - 6
	ADDQ.w	#6,d0				* add the minimum number of moves for level 1
	NEG.w		d0				* make negative
	ADD.w		moves_made(a3),d0		* calculate the number of extra moves taken
	MOVE.w	#11,moves_made(a3)	* set the number of extra moves taken to lose
	CMPI.w	#10,d0			* compare with the maximum moves for a win
	BGT.s		was_not_win			* if not a win skip the level increment

	MOVE.w	d0,moves_made(a3)		* else save the number of extra moves taken
	MOVE.l	#0,last_board(a3)		* clear the last game board

	CMP.w		#1,select_mode(a3)	* compare the select mode with select game
	BNE.s		was_not_win			* if we are not playing the level games skip
							* the level increment

	ADDQ.w	#1,play_level(a3)		* increment the play level
was_not_win
	MOVE.w	#mode_win,game_mode(a3)	* set the game mode to win mode
skip_win
	MOVE.w	#0,button_l(a3)		* clear the left button flag
	MOVE.l	game_board(a3),d0		* get the board
	CMP.l		display_board(a3),d0	* compare it with the display
	BEQ.s		exit_play_mode		* if not changed just exit

	MOVE.l	d0,display_board(a3)	* else save the display
	MOVE.w	#-1,draw_board(a3)	* and set the draw board flag
exit_play_mode
	RTS


*************************************************************************************
*
* get the minimum number of moves - 6 for this level

get_moves
	CMPI.w	#50,d0			* compare with 50, which is level 51
	BNE.s		is_00to49			* skip reset if level was 1 to 50

	MOVEQ		#49,d0			* else make the level 49
is_00to49
	DIVU.w	#5,d0				* convert level to 0 to 9
	RTS


*************************************************************************************
*
* does the chase lights bit if you win in the minimum number of moves moves. if more
* than the minimum number of moves is taken this flashes the count of extra moves
* made. if more than ten extra moves were made a X pattern is flashed.

win_mode
	TST.w		chase_seq(a3)		* test if the sequence has started
	BPL.s		no_intro			* if so skip the intro

	BSR		play_intro			* play the intro noise
no_intro
	PEA		win_next(pc)		* push the return address
	TST.w		moves_made(a3)		* test the number of extra moves made
	BNE		score_flash			* if extra moves go do the extra moves flash

	BRA		lights_chase		* go do the lights chase

* whichever lights routine was taken program flow will return to here because of the
* PEA at the start of this subroutine

win_next
	TST.l		d1				* test if the sequence is complete
	BEQ.s		exit_win_mode		* if not just exit

	MOVE.w	#mode_start_play,game_mode(a3)
							* else set the game mode to start game mode
	MOVE.w	#0,button_l(a3)		* clear the left button flag
exit_win_mode
	RTS


*************************************************************************************
*
* select mode

* the select mode will either display the current select mode or the selected level.
* the level will only be displayed after a button has been pressed. pressing select
* will always make the display show the current mode, pressing a light button will
* select that level for play if the select mode is random

selct_mode
	BSR		control_buttons		* handle the control buttons
	MOVE.w	#0,button_l(a3)		* clear the left button flag

	MOVEQ		#0,d0				* clear the longword
	MOVE.w	select_mode(a3),d0	* get the select mode
	CMP.l		display_board(a3),d0	* compare it with the display
	BEQ.s		exit_selct_mode		* if not changed just exit

	MOVE.l	d0,display_board(a3)	* else save the display
	MOVE.w	#-1,draw_board(a3)	* and set the draw board flag
exit_selct_mode
	RTS


*************************************************************************************
*
* flash the help hint

help_hint_mode
	LEA		solution(a3),a0		* point to the solution longword pair
	BRA.s		flash_mode			* go flash the display


*------------------------------------------------------------------------------------
*
* flash the '+' to show the file saved

save_mode
	LEA		save_flash(pc),a0		* point to the save flash pattern


*------------------------------------------------------------------------------------
*
* flash the pattern pointed to by (a0)

flash_mode
	BSR		lights_flash		* do the lights flash
	TST.l		d1				* test if the sequence is complete
	BEQ.s		exit_flash_mode		* if not just exit

	MOVE.w	last_mode(a3),game_mode(a3)
							* else restore the game mode
	MOVE.w	#0,button_l(a3)		* clear the left button flag
exit_flash_mode
	RTS


*************************************************************************************
*
* setup stuff

initialise
	MOVEQ		#50,d0			* close all files
	TRAP		#15

	MOVEQ		#17,d1			* enable double buffering
	MOVEQ		#92,d0			* set draw mode
	TRAP		#15

	LEA		variables(pc),a3		* set the base pointer to the variables

	BSR		mouse_init			* initialise the mouse interrupt and flags
	BSR		sound_init			* load the sound effects
	BSR		load_play			* get the play level

	MOVE.w	#xyoff-x_tl,im_x(a3)	* set the image x origin
	MOVE.w	#xyoff-y_tl,im_y(a3)	* set the image y origin
	MOVE.w	#mode_welcome,game_mode(a3)
							* set game mode to welcome mode
	MOVE.w	#-1,play_sound(a3)	* set the play sound flag
	MOVE.w	#-1,chase_seq(a3)		* reset the flash/sequence state
	MOVE.l	#0,last_board(a3)		* clear the last game board
	RTS


*************************************************************************************
*
* load the play level

load_play
	LEA		settings_name(pc),a1	* point to the settings file name
	MOVEQ		#51,d0			* open existing file
	TRAP		#15

	TST.w		d0				* test the result
	BNE.s		no_settings			* if not open go set the defaults

	LEA		PRNlword(a3),a1		* point to the random number key
	MOVEQ		#4,d2				* set the bytes to read count
	MOVEQ		#53,d0			* read file
	TRAP		#15

	TST.w		d0				* test the result
	BNE.s		no_settings			* if read error go set the defaults

	LEA		select_mode(a3),a1	* point to the select mode
	MOVEQ		#4,d2				* set the bytes to read count
	MOVEQ		#53,d0			* read file
	TRAP		#15

	TST.w		d0				* test the result
	BNE.s		no_settings			* if read error go set the defaults

	MOVEQ		#56,d0			* close file
	TRAP		#15

	BSR		gen_prng			* generate a new random number
	MOVE.l	PRNlword(a3),d0		* get a random word
	MOVE.l	select_mode(a3),d1	* get the select mode and play level
	EOR.l		d0,d1				* unscramble the select mode and play level

	CMP.w		#50,d1			* compare with max level
	BCC.s		default_settings		* if > 50 go use the defaults

	MOVE.l	d1,select_mode(a3)	* save the select mode and play level
	RTS

no_settings
	MOVEQ		#50,d0			* close all files
	TRAP		#15

default_settings
	MOVE.l	#$8C9F53D0,PRNlword(a3)	* set PRNG seed. this can be any value
							* except zero, changing this will produce
							* a different start point for the generator
	MOVE.l	#$00010000,select_mode(a3)
							* set the select mode and play level
	RTS


*------------------------------------------------------------------------------------
*
* the settings file name

settings_name
	dc.b	'lightsout.dat',0			* settings file name
	ds.w	0					* ensure even


*------------------------------------------------------------------------------------
*
* save the play level

save_play
	LEA		settings_name(pc),a1	* point to the settings file name
	MOVEQ		#52,d0			* open new file
	TRAP		#15

	TST.w		d0				* test the result
	BNE.s		not_saved			* if not open close all files and exit

	LEA		PRNlword(a3),a1		* point to the random number key
	MOVEQ		#4,d2				* set the bytes to save count
	MOVEQ		#54,d0			* write file
	TRAP		#15

	TST.w		d0				* test the result
	BNE.s		not_saved			* if write error close all files and exit

	BSR		gen_prng			* generate a new random number
	MOVE.l	PRNlword(a3),d0		* get a random word
	MOVE.l	select_mode(a3),d3	* get the select mode and play level
	EOR.l		d0,select_mode(a3)	* scramble the select mode and play level

	LEA		select_mode(a3),a1	* point to the select mode and play level
	MOVEQ		#4,d2				* set the bytes to save count
	MOVEQ		#54,d0			* write file
	TRAP		#15

	MOVE.l	d3,select_mode(a3)	* restore the select mode and play level

	TST.w		d0				* test the result
	BNE.s		not_saved			* if read error go set the defaults

	MOVEQ		#56,d0			* close file
	TRAP		#15

	MOVE.w	game_mode(a3),last_mode(a3)
							* save the game mode
	MOVE.w	#mode_save,game_mode(a3)
							* set the game mode to save mode
	RTS

not_saved
	MOVEQ		#50,d0			* close all files
	TRAP		#15

	RTS


*************************************************************************************
*
* draw the board, this draws the disply board not the game board as the two may be
* different depending on the game state and effects being used

do_draw
	TST.w		draw_board(a3)		* test the draw board flag
	BEQ		exit_do_draw		* if not set just exit

	MOVEQ		#0,d1				* pen colour to black
	MOVEQ		#80,d0			* set the pen colour
	TRAP		#15

	MOVEQ		#1,d7				* set bit 0 index
	MOVEQ		#4,d6				* set row count
yloop
	MOVEQ		#4,d5				* set column count
xloop
	MOVE.l	off(pc),d1			* set the fill colour to off
	MOVE.l	d7,-(sp)			* save the bit index
	AND.l		display_board(a3),d7	* mask this bit
	BEQ.s		is_off			* if square is off skip colour on set

	MOVE.l	on(pc),d1			* set the fill colour to on
is_off
	MOVE.l	(sp)+,d7			* restore the bit index
	ADD.l		d7,d7				* shift for next bit

	MOVEQ		#81,d0			* set the fill colour
	TRAP		#15

* now calculate x1,y1 and x2,y2 from the button column and row

	MOVEQ		#4,d1				* set x1 max
	MOVEQ		#4,d2				* set y1 max
	SUB.w		d5,d1				* calculate x1
	SUB.w		d6,d2				* calculate y1
	MULU.w	#xstep,d1			* scale x1
	MULU.w	#ystep,d2			* scale y1
	MOVEQ		#xyoff,d4			* x and y offset
	ADD.l		d4,d1				* offset x1
	ADD.l		d4,d2				* offset y1
	MOVEQ		#xwide,d3			* offset to x2
	MOVEQ		#ywide,d4			* offset to y2
	ADD.w		d1,d3				* calculate x2
	ADD.w		d2,d4				* calculate y2

	MOVEQ		#87,d0			* draw a filled rectangle
	TRAP		#15

	DBF		d5,xloop			* do the next column

	DBF		d6,yloop			* do the next row

	MOVEQ		#94,d0			* copy the screen buffer to main
	TRAP		#15

	MOVE.w	#0,draw_board(a3)		* clear the draw board flag
exit_do_draw
	RTS


*************************************************************************************
*
* check the control buttons

control_buttons
	TST.w		button_l(a3)		* test the left mouse button flag
	BEQ.s		exit_control_buttons	* if not set just exit

	BSR		get_control_button	* get the control button number
	TST.w		d0				* test the result
	BEQ.s		exit_control_buttons	* if invalid skip the button action

	LEA		control_action(pc),a0	* point to the action table
	JSR		(a0,d7.w)			* go do the action

	BSR		play_click			* play the click noise
	MOVE.w	#0,button_l(a3)		* clear the left button flag
exit_control_buttons
	RTS


*------------------------------------------------------------------------------------
*
* control button action branch table

control_action
	BRA.w		button_select		* select button action
	BRA.w		button_save			* save button action
	BRA.w		button_start		* start button action
	BRA.w		button_sound		* sound button action
	BRA.w		button_help			* help button action


*------------------------------------------------------------------------------------
*
* the select button drops the game into select mode. if the game is already in select
* mode then the mode is toggled between the standard fifty levels play and random
* game play

button_select
	MOVEQ		#3,d0				* set select mode toggle
	MOVEQ		#mode_select,d1		* set select mode
	CMP.w		game_mode(a3),d1		* is the mode already select mode
	BNE.s		no_toggle_select		* if not skip toggling the select mode

	EOR.w		d0,select_mode(a3)	* else toggle the select mode
no_toggle_select
	MOVE.w	d1,game_mode(a3)		* set the game mode to select mode
	MOVE.l	#0,last_board(a3)		* clear the last game board
	RTS


*------------------------------------------------------------------------------------
*
* the save button saves the current level and mode to the settings file

button_save
	BRA		save_play			* go save the play level


*------------------------------------------------------------------------------------
*
* the start button starts a puzzle or restarts a puzzle in progress

button_start
	MOVEQ		#mode_play,d0		* set for play game
	CMP.w		game_mode(a3),d0		* test the current game mode
	BEQ		start_game			* if playing just restart the game

	BSR		play_start			* play the start noise
	MOVE.w	#mode_start_play,game_mode(a3)
							* else set the game mode to start game mode
	RTS


*------------------------------------------------------------------------------------
*
* the sound button toggles the sound off and on

button_sound
	MOVEQ		#-1,d0			* set to toggle flag
	EOR.w		d0,play_sound(a3)		* toggle the play sound flag
	RTS


*------------------------------------------------------------------------------------
*
* display the puzzle help. the solution to the current board layout is calculated
* and one bit from that solution is randomly selected as the help. if help is pressed
* again without moving that help is repeated. Each time a new help is used the
* remaining help available is reduced

button_help
	CMP.w		#mode_play,game_mode(a3)
							* test the game mode
	BNE.s		exit_button_help		* if not play mode just exit

	TST.l		solution(a3)		* see if a hint is present
	BNE.s		flash_button_help		* if there is just repeat it

	TST.w		help_count(a3)		* test the help count
	BEQ.s		exit_button_help		* if no help left just exit

* help is needed so first solve the board. this is done by calculating the solution
* from a solution matrix that holds the solution to the twenty three unique patterns

	MOVE.l	display_board(a3),d0	* get the board pattern
	LEA		solution_matrix(pc),a0	* point to the solution matrix
	MOVEQ		#0,d1				* clear the solution
	MOVEQ		#22,d7			* set the bit count
solve_loop
	MOVE.l	(a0)+,d3			* get the solution pattern longword
	LSR.l		#1,d0				* shift the board pattern
	BCC.s		not_this_lamp		* if this lamp is off skip the pattern add

	EOR.l		d3,d1				* else and add it to the solution
not_this_lamp
	DBF		d7,solve_loop		* loop if more to do

* found a solution, now check to see it is the minimal solution by adding in the three
* null space terms and chosing the shortest result

	SUBQ.w	#6,sp				* make room on the stack for the solution
	MOVEQ		#0,d3				* clear the EOR term
	BSR		count_one			* get the count for this solution, no term load
	BSR		count_ones			* get the ones count for this solution
	BSR		count_ones			* get the ones count for this solution
	SUBQ.w	#8,a0				* decrement the pointer to the last null pattern
	BSR		count_ones			* get the ones count for this solution

	ADDQ.w	#2,sp				* dump the bit count
	MOVE.l	(sp)+,d1			* retrieve the shortest solution

* now to chose one random bit from all the solution bits

	MOVEQ		#25,d0			* set for bit 0 to 24
	BSR		get_prng			* get a random number in the range 0 to d0 - 1

	MOVEQ		#24,d7			* set the bit count
bit_test_loop
	BTST		d0,d1				* test the bit in the solution
	BNE.s		found_bit			* if this bit is set exit

	SUBQ.w	#1,d0				* else decrement the bit number
	BPL.s		not_rolled_out		* if still in the solution skip the set

	MOVEQ		#24,d0			* set the top bit of the solution space
not_rolled_out
	DBF		d7,bit_test_loop		* loop if more to do

* use this bit as the solution help bit

found_bit
	MOVE.l	display_board(a3),d1	* get the board pattern
	MOVE.l	d1,d3				* copy the board pattern
	BSET		d0,d1				* set the on bit
	BCLR		d0,d3				* clear the off bit
	LEA		8+solution(a3),a0		* point to the solution longword pair
	MOVEM.l	d1/d3,-(a0)			* save the solution help patterns

	SUBQ.w	#1,help_count(a3)		* decrement the help count
flash_button_help
	MOVE.w	#mode_play,last_mode(a3)
							* set the last mode to game mode
	MOVE.w	#mode_help_hint,game_mode(a3)
							* set the game mode to help hint mode
exit_button_help
	RTS


*************************************************************************************
*
* get the next EOR longword and count the number of ones in d1.l, return the count in
* d0.w

count_ones
	MOVE.l	(a0)+,d3			* get the next null space pattern longword
count_one
	EOR.l		d1,d3				* add to the solution
	MOVE.l	d3,d1				* copy the new solution

	MOVEQ		#24,d7			* set the bit count
	MOVEQ		#0,d0				* clear the ones total
count_ones_loop
	LSR.l		#1,d3				* shift bit to carry
	BCC.s		not_this_one		* if not a one skip the total increment

	ADDQ.w	#1,d0				* increment the ones total
not_this_one
	DBF		d7,count_ones_loop	* loop if more to do

	CMP.w		4(sp),d0			* compare this ones count with the last
	BCC.s		skip_second			* if the first solution is better don't swap

	MOVE.w	d0,4(sp)			* save the count
	MOVE.l	d1,6(sp)			* save the new solution
skip_second
	RTS


*************************************************************************************
*
* check the light buttons. if the left button has been pressed this routine clears
* the button flag and checks if it was a valid light button that was pressed

light_buttons
	TST.w		button_l(a3)		* test the left mouse button flag
	BEQ.s		exit_l_buttons		* if not set go test the right button flag

	MOVE.w	#0,button_l(a3)		* clear the left mouse button flag
	BSR		light_button		* get the light button number
	TST.w		d0				* test the result
	BEQ.s		exit_l_buttons		* if invalid skip the light button action

	LEA		button_table(pc),a0	* point to the EOR table
	ADD.w		d1,d1				* * 2
	ADD.w		d1,d1				* * 4 for longword index
	MOVE.l	(a0,d1.w),d0		* get the EOR table value
	EOR.l		d0,game_board(a3)		* toggle the board bit
	ADDQ.w	#1,moves_made(a3)		* increment the moves made count

	BSR		play_click			* play the click noise
	MOVE.l	#0,solution(a3)		* clear the solution longword
exit_l_buttons
	RTS


*************************************************************************************
*
* gets the light button index, 0 to 24, from mouse x,y, returns d1.w = button, d0.l
* = -1 if valid, = 0 if not valid

light_button
	MOVEQ		#0,d0				* flag invalid
	MOVEQ		#0,d1				* clear x longword
	MOVEQ		#0,d2				* clear y longword
	MOVE.w	mousex(a3),d1		* get mouse x
	MOVE.w	mousey(a3),d2		* get mouse y
	MOVEQ		#xyoff,d4			* x and y offset
	SUB.l		d4,d1				* subtract offset from x
	BMI.s		exit_light_button		* exit if x was < minimum

	SUB.l		d4,d2				* subtract offset from y
	BMI.s		exit_light_button		* exit if y was < minimum

	DIVU.w	#xstep,d1			* scale x
	CMPI.w	#4,d1				* compare result with max + 1
	BGT.s		exit_light_button		* exit if x was > maximum

	SWAP		d1				* get the remainder form the DIVU
	CMPI.w	#xwide,d1			* compare result with button x width
	BGT.s		exit_light_button		* exit if x was > maximum

	SWAP		d1				* restore the button x number

	DIVU.w	#ystep,d2			* scale y
	CMPI.w	#4,d2				* compare result with max + 1
	BGT.s		exit_light_button		* exit if y was > maximum

	SWAP		d2				* get the remainder form the DIVU
	CMPI.w	#ywide,d2			* compare result with button y width
	BGT.s		exit_light_button		* exit if y was > maximum

	SWAP		d2				* restore the button y number

	MULU.w	#5,d2				* y * 5
	ADD.w		d2,d1				* add to x
	MOVEQ		#-1,d0			* flag valid
exit_light_button
	RTS


*************************************************************************************
*
* gets a control button index. 25 to 29 for the select, reset, start, sound and help
* buttons. returns d7.w = button, d0.l = -1 if valid, = 0 if not valid

get_control_button
	MOVEQ		#0,d0				* flag invalid
	MOVE.w	mousex(a3),d1		* get mouse x
	MOVE.w	mousey(a3),d2		* get mouse y

	SUB.w		im_x(a3),d1			* subtract image offset x from x
	SUB.w		im_y(a3),d2			* subtract image offset y from y

	LEA		control_b_table-8(pc),a0
							* point to the buttons table
	MOVEQ		#-4,d7			* set the index for the first increment
get_c_button_loop
	ADDQ.w	#4,d7				* increment the button index
	ADDQ.w	#8,a0				* increment the table index

	MOVE.w	(a0),d4			* get x1
	BMI.s		exit_get_control_button	* if it's the end of the table, exit

	CMP.w		d4,d1				* compare x with x1
	BLT.s		get_c_button_loop		* if < x1 go try the next button

	CMP.w		4(a0),d1			* compare x with x2
	BGT.s		get_c_button_loop		* if > x2 go try the next button

	CMP.w		2(a0),d2			* compare y with y1
	BLT.s		get_c_button_loop		* if < y1 go try the next button

	CMP.w		6(a0),d2			* compare y with y2
	BGT.s		get_c_button_loop		* if > y2 go try the next button

* x,y is within the range of button d7 so return the result	

	MOVEQ		#-1,d0			* flag valid
exit_get_control_button
	RTS

*------------------------------------------------------------------------------------
*
* other buttons table. values are x1,y1,x2,y2 where x1 < x2 and y1 < y2. co-ordinates
* are those of the image not the screen

control_b_table
	dc.w	34,282,93,302			* select	25
	dc.w	131,233,156,242			* save	26
	dc.w	131,260,156,269			* start	27
	dc.w	131,287,156,296			* sound	28
	dc.w	131,314,156,323			* help	29
	dc.w	-1					* end marker


*************************************************************************************
*
* does the start of the game flash, returns d1.l = -1 when complete, else d1.l = 0

start_flash
	MOVE.w	play_level(a3),d0		* get the play level
	CMPI.w	#2,select_mode(a3)	* compare the select mode with random play
	BEQ.s		do_lights_flash		* if random play go start a game

	CMPI.w	#50,d0			* compare the play level with match over
	BEQ.s		done_lights_flash		* if match over just exit

do_lights_flash
	MULU.w	#8,d0				* multiply by the bytes per pattern pair
	LEA		start_table(pc),a0	* point to the start pattern table
	LEA		(a0,d0.w),a0		* get the pattern address for this level
	BRA.s		lights_flash		* go flash the lighs


*------------------------------------------------------------------------------------
*
* flashes the score for the end of the game, returns d1.l = -1 when complete, else
* d1.l = 0

score_flash
	MOVEQ		#8,d0				* bytes per pattern longword pair
	MULU.w	moves_made(a3),d0		* multiply by the number of extra moves taken
	LEA		flash_table(pc),a0	* point to the flash pattern table
	LEA		(a0,d0.w),a0		* get the pattern address for these moves
*##	BRA.s		lights_flash		* go flash the lighs


*------------------------------------------------------------------------------------
*
* flashes the lights four times, enter with the pattern pair pointer in a0. returns
* d1.l = -1 when complete, else d1.l = 0

lights_flash
	MOVE.w	chase_seq(a3),d7		* get the chase sequence number
	BPL.s		not_flash_start		* if running skip the sequence start

	MOVEQ		#28,d7			* set the light sequence to the start
	MOVE.w	d7,chase_seq(a3)		* set the light sequence #
	MOVE.w	#2,chase_loop(a3)		* reset the chase loop count
not_flash_start
	ANDI.w	#4,d7				* mask for an offset of 0 or 4
	MOVE.l	(a0,d7.w),display_board(a3)
							* set the display from the flash pattern
	MOVE.w	#-1,draw_board(a3)	* set the draw board flag

	MOVEQ		#0,d1				* flag not done yet
	SUBQ.w	#1,chase_loop(a3)		* decrement the chase loop count
	BPL.s		exit_lights_flash		* if not done yet just exit

	MOVE.w	#2,chase_loop(a3)		* reset the chase loop count
	SUBQ.w	#4,chase_seq(a3)		* decrement the sequence count
	BPL.s		exit_lights_flash		* if not done yet don't flag done

done_lights_flash
	MOVEQ		#-1,d1			* flag sequence complete
exit_lights_flash
	RTS


*************************************************************************************
*
* does the round the board chase sequence, returns d1.l = -1 when complete, else d1.l
* = 0

lights_chase
	MOVE.w	chase_seq(a3),d7		* get the chase sequence number
	BPL.s		not_start_chase		* if started skip the sequence reset

	BSR		play_click			* play the click noise
	MOVEQ		#96,d7			* set the sequence to the start
	MOVE.w	d7,chase_seq(a3)		* set the chase sequence
	MOVE.w	#1,chase_loop(a3)		* reset the chase loop count
not_start_chase
	LEA		chase_table(pc),a0	* point to the chase sequence table
	MOVE.l	(a0,d7.w),display_board(a3)
							* set the display from the flash pattern
	MOVE.w	#-1,draw_board(a3)	* set the draw board flag

	MOVEQ		#0,d1				* flag not done yet
	SUBQ.w	#1,chase_loop(a3)		* decrement the chase loop count
	BPL.s		exit_lights_chase		* if not done yet just exit

	BSR		play_click			* play the click noise
	MOVE.w	#1,chase_loop(a3)		* reset the chase loop count
	SUBQ.w	#4,chase_seq(a3)		* and decrement the sequence loop count
	BPL.s		exit_lights_chase		* if not done yet don't flag done

	MOVEQ		#-1,d1			* flag sequence complete
exit_lights_chase
	RTS


*************************************************************************************

	INCLUDE	mouse.x68
							* mouse routines
	INCLUDE	sounds.x68
							* sound routines
	INCLUDE	random.x68
							* prng routines
	INCLUDE	background.x68
							* bitmap load routines


*************************************************************************************
*
* program constants and tables

* solution matrix for the board. there are only 23 entries because the state of the
* last two buttons is forced by the pattern on the previous 23

solution_matrix
	dc.l	$08628E,$04705B,$2FB1BD,$710047,$3A5436
	dc.l	$0C12D4,$23A36A,$5AC1A5,$64E5C4,$4B5461
	dc.l	$27D310,$710000,$323636,$047147,$15E793
	dc.l	$5AC5C4,$474E4C,$1D96D4,$476526,$5A82B5
	dc.l	$3AD898,$701D5C,$2D8B88

* null space patterns. none, either or both of these combined with a solution from the
* solution matrix give the minimal solution

	dc.l	$0EAEEAE				* hash null pattern
	dc.l	$15A82B5				* star null pattern

* the help counts for the different levels. lots of help at first but tails off quickly
* then more help for the later levels

level_help
	dc.w	6,6,5,4,3,2,2,2,2,2		* levels  1 to 10
	dc.w	2,2,2,2,2,2,2,2,2,2		* levels 10 to 20
	dc.w	2,2,2,2,2,3,3,3,3,3		* levels 20 to 30
	dc.w	3,3,4,4,4,4,4,4,4,5		* levels 30 to 40
	dc.w	5,5,5,5,5,6,6,6,6,6		* levels 40 to 50

* off/on colours

off
	dc.l	$00C080A0				* the colour when a button is off
on
	dc.l	$005818F8				* the colour when a button is on

* score flash pattern data

flash_table					EQU	*-8	* index starts at 1
	dc.l	$00000001,$00000000		*  1 over minimum
	dc.l	$00000003,$00000000		*  2 over minimum
	dc.l	$00000007,$00000000		*  3 over minimum
	dc.l	$0000000F,$00000000		*  4 over minimum
	dc.l	$0000001F,$00000000		*  5 over minimum
	dc.l	$0000003F,$00000000		*  6 over minimum
	dc.l	$0000007F,$00000000		*  7 over minimum
	dc.l	$000000FF,$00000000		*  8 over minimum
	dc.l	$000001FF,$00000000		*  9 over minimum
	dc.l	$000003FF,$00000000		* 10 over minimum
	dc.l	$01151151,$00000000		* 11 or more over minimum

* game start pattern data

start_table
	dc.l	$00000001,$00000000		* level  1
	dc.l	$00000002,$00000000		* level  2
	dc.l	$00000004,$00000000		* level  3
	dc.l	$00000008,$00000000		* level  4
	dc.l	$00000010,$00000000		* level  5
	dc.l	$00000020,$00000000		* level  6
	dc.l	$00000040,$00000000		* level  7
	dc.l	$00000080,$00000000		* level  8
	dc.l	$00000100,$00000000		* level  9
	dc.l	$00000200,$00000000		* level 10
	dc.l	$00000400,$00000000		* level 11
	dc.l	$00000800,$00000000		* level 12
	dc.l	$00001000,$00000000		* level 13
	dc.l	$00002000,$00000000		* level 14
	dc.l	$00004000,$00000000		* level 15
	dc.l	$00008000,$00000000		* level 16
	dc.l	$00010000,$00000000		* level 17
	dc.l	$00020000,$00000000		* level 18
	dc.l	$00040000,$00000000		* level 19
	dc.l	$00080000,$00000000		* level 20
	dc.l	$00100000,$00000000		* level 21
	dc.l	$00200000,$00000000		* level 22
	dc.l	$00400000,$00000000		* level 23
	dc.l	$00800000,$00000000		* level 24
	dc.l	$01000000,$00000000		* level 25

	dc.l	$01000001,$01000000		* level 26
	dc.l	$01000002,$01000000		* level 27
	dc.l	$01000004,$01000000		* level 28
	dc.l	$01000008,$01000000		* level 29
	dc.l	$01000010,$01000000		* level 30
	dc.l	$01000020,$01000000		* level 31
	dc.l	$01000040,$01000000		* level 32
	dc.l	$01000080,$01000000		* level 33
	dc.l	$01000100,$01000000		* level 34
	dc.l	$01000200,$01000000		* level 35
	dc.l	$01000400,$01000000		* level 36
	dc.l	$01000800,$01000000		* level 37
	dc.l	$01001000,$01000000		* level 38
	dc.l	$01002000,$01000000		* level 39
	dc.l	$01004000,$01000000		* level 40
	dc.l	$01008000,$01000000		* level 41
	dc.l	$01010000,$01000000		* level 42
	dc.l	$01020000,$01000000		* level 43
	dc.l	$01040000,$01000000		* level 44
	dc.l	$01080000,$01000000		* level 45
	dc.l	$01100000,$01000000		* level 46
	dc.l	$01200000,$01000000		* level 47
	dc.l	$01400000,$01000000		* level 48
	dc.l	$01800000,$01000000		* level 49
	dc.l	$018C0000,$01000000		* level 50
	dc.l	$018C0000,$01000000		* level 51 for mode two play

* chase sequence table

chase_table
	dc.l	$00001000,$00000080,$00000100,$00002000,$00040000
	dc.l	$00020000,$00010000,$00000800,$00000040,$00000002
	dc.l	$00000004,$00000008,$00000010,$00000200,$00004000
	dc.l	$00080000,$01000000,$00800000,$00400000,$00200000
	dc.l	$00100000,$00008000,$00000400,$00000020,$00000001

* button EOR table

button_table
	dc.l	$00000023,$00000047,$0000008E,$0000011C,$00000218
	dc.l	$00000461,$000008E2,$000011C4,$00002388,$00004310
	dc.l	$00008C20,$00011C40,$00023880,$00047100,$00086200
	dc.l	$00118400,$00238800,$00471000,$008E2000,$010C4000
	dc.l	$00308000,$00710000,$00E20000,$01C40000,$01880000

* 50 puzzle table. the first five puzzles take six moves, then seven etc until the last
* five that take fifteen moves

fifty_puzzle
	dc.l	$00005400,$015A82B5,$00ADEF6A,$01B88360,$01BC5EEF	*  6 move puzzles
	dc.l	$00EAD400,$00F8C62F,$00AAA880,$007D3BEA,$000039CE	*  7 move puzzles
	dc.l	$00EAD6B5,$00A76D5F,$0022AA88,$00210800,$00010040	*  8 move puzzles
	dc.l	$01F08421,$01F71000,$00455544,$01505415,$00004400	*  9 move puzzles
	dc.l	$0021385E,$00E8C62E,$00467000,$012FC400,$01E79C61	* 10 move puzzles
	dc.l	$0118FE31,$004211C4,$01CE7000,$00000040,$00001000	* 11 move puzzles
	dc.l	$011CD671,$01F1111F,$0198D508,$01EB4634,$000AC558	* 12 move puzzles
	dc.l	$011FC544,$000739C0,$01555555,$00A60C2A,$00002800	* 13 move puzzles
	dc.l	$00421151,$00749D27,$00E11D71,$00E27F60,$0157F0AE	* 14 move puzzles
	dc.l	$00477DC4,$010917E4,$00089220,$01151151,$01FFFFFF	* 15 move puzzles

* flash pattern for the settings save, also the "+" pattern for when level 50 is
* completed. this must directly follow the 50 puzzle table

save_flash
	dc.l	$00023880								* + pattern
	dc.l	$00000000								* all off

* puzzle data for the mode 2, random puzzle, game mode

random_puzzle
	dc.l	$0056BABD,$00A535D2,$01FBB1EF,$005AC8B7,$0043C128	* 6 move puzzles
	dc.l	$014E573E,$00861724,$016C9693,$01B2E5F3,$01ECF6FF
	dc.l	$00D9D0B9,$01EA373E,$00973636,$00D2D0EF,$0037A1F0
	dc.l	$00545E6D,$006AFC14,$012D0FB5,$00C8D9A6,$00FAD8D4
	dc.l	$00643828,$01B2E9DD,$00D3DD6B,$00D9C365,$0076EDC4
	dc.l	$00F3A5F4,$01DC0F9E,$00828FE6,$00C1CE2A,$0082C414
	dc.l	$011F0A4B,$0006D282,$00B9484F,$00F8E1B3,$00D22DF7
	dc.l	$01172904,$012664DA,$00C94CBB,$00E9554A,$018F0364
	dc.l	$005AA974,$0049F256,$0032675A,$00FBECD9,$01ACE242
	dc.l	$00B08F65,$0145A33F,$012D0FD1,$01B52D13,$00A3CAEE
	dc.l	$01E62BDD,$017FA5EF,$0142DFC7,$0049E874,$0068EC74
	dc.l	$00D9D569,$0160EE1B,$00A5D5CA,$0058675B,$005AE49B
	dc.l	$0082C169,$01CF57DC,$0056ABF7,$00177F3E,$01EC3FD2
	dc.l	$00FBF1BD,$01DC6D90,$00FB68B6,$00C7C392,$00205E2D
	dc.l	$00DB4B04,$013B8DB0,$01B06F7B,$01B58B45,$01F89726
	dc.l	$00F7B6BD,$006271A8,$002A4E9B,$0049A082,$01BBF2AD
	dc.l	$0168B1EF,$0093CE0F,$016D9BDE,$00FF963F,$001626DE
	dc.l	$00FA48EF,$01BC7E24,$01A4C586,$00A7574A,$002858D4
	dc.l	$003EDCB4,$00FA5D4A,$005677B6,$005466DC,$002E3164
	dc.l	$00DBFEBB,$00CA7C28,$01EE064E,$01AF820C,$0032EF7F

	dc.l	$017AEE93,$01293CB9,$0160FFBB,$00F2FEA1,$0091F6EB	* 7 move puzzles
	dc.l	$01E502E6,$012565EB,$00984696,$01E887FF,$0192D56C
	dc.l	$01E11516,$0068E0B0,$0186B068,$01C5A0D9,$0142C66F
	dc.l	$00E90774,$00191159,$011645CA,$00AF2322,$002BD25B
	dc.l	$00966A25,$01E2DF8D,$0176A2E7,$00D8D35D,$01243281
	dc.l	$01D6B7E2,$00C1EAE4,$0074DE02,$0151EDA7,$00EEA77B
	dc.l	$01B3B791,$00F797A3,$01FCFB8B,$01DA3962,$01BDFEBF
	dc.l	$0152DEAF,$00F8491A,$0156AD65,$00CBFAF8,$0074F642
	dc.l	$01ADF8D6,$00D05CEF,$002A054A,$0165EF1B,$0056FFB0
	dc.l	$00D4D9FF,$00BF5DB2,$006F6B7F,$0083F77B,$012526EA
	dc.l	$015ECB7B,$01112374,$0173BBDD,$01A13BDD,$00D6F622
	dc.l	$00D0CCB0,$00DDF42B,$00C3F2E8,$005844D3,$0106AE64
	dc.l	$00582482,$0122C242,$006A0764,$01DE5EE0,$0006F035
	dc.l	$008FD9D3,$01526F04,$00FBC7C6,$01A0DD4A,$00CAB71D
	dc.l	$01277837,$01E34B66,$0069D109,$00A2D929,$002C48DE
	dc.l	$013BAB45,$00D97144,$00F4D13B,$013AFD93,$01EE8B35
	dc.l	$01DE77D2,$00581A4A,$006D0875,$003B69B0,$01FB3A24
	dc.l	$00B84BFD,$00D96058,$00F8706F,$00417D3B,$00F1C1FF
	dc.l	$00544554,$006E5541,$011EC949,$01A4D4EF,$01AB6EBE
	dc.l	$0142E5A7,$0194DFBB,$00C11CF7,$006C8452,$01F5DEE1

	dc.l	$01C83ABC,$01E3EB12,$01A71CB9,$015EE85E,$00CDB5D9	* 8 move puzzles
	dc.l	$00DD8673,$00D994A8,$0046A855,$01E608C7,$00F85533
	dc.l	$01DB290C,$012A256A,$0105D44F,$001CE31E,$0137C4B9
	dc.l	$0000F6E8,$01F8B050,$01EDEE25,$01425482,$00B12E4A
	dc.l	$0157B1F7,$0125618A,$0000F608,$00092B64,$01737105
	dc.l	$00951B30,$00C45DDC,$00DC80FC,$01135EDA,$0142DE55
	dc.l	$00C9769B,$01DB7496,$01597F7B,$01A0E7E3,$003B2764
	dc.l	$01077ACD,$00C4D59D,$01BFE4AB,$0177D269,$017318D5
	dc.l	$01FBFA1D,$00F87CFA,$006D2E2D,$008F89E4,$0012CF7F
	dc.l	$001BB722,$01B3FAF0,$013B4EB2,$005BC468,$00D95D28
	dc.l	$01AD0E34,$00C16D65,$00AA1A5A,$00962935,$01458BFC
	dc.l	$00FADC3B,$012C573C,$00788EBF,$00ECB5FF,$01C3F508
	dc.l	$00FC280B,$00972077,$00F96FAF,$00F254FA,$006507E0
	dc.l	$00A94E8B,$01A96FAB,$0148AA25,$012F4A75,$01AA789B
	dc.l	$0119073A,$00C2AE64,$00F8C49A,$001739F4,$019EBAEF
	dc.l	$004F643E,$0151FBBA,$008BF61B,$005C39FA,$01F22AB2
	dc.l	$00647C8F,$005A2358,$0071A8FF,$01604589,$01735646
	dc.l	$008FFF42,$00A3689E,$01437580,$0172E7B6,$000E9677
	dc.l	$00974C8B,$01FBF3E3,$00FE14EA,$003E13E0,$00CA6455
	dc.l	$01FBB709,$0133287E,$004D9EBC,$0197E7CA,$00EA5E97

	dc.l	$01C83EDD,$00AD618A,$00535112,$00A3579F,$01F3BC88	* 9 move puzzles
	dc.l	$004F8F91,$00FE96ED,$01B7398D,$000892A3,$01A36C37
	dc.l	$000AE3FC,$0113ABA3,$01AF4F72,$012561CD,$00B88CC5
	dc.l	$0068B5F7,$016308F7,$01F392D5,$01BC2050,$0102DADA
	dc.l	$0169068C,$002CF077,$0033DC0B,$0018694D,$00997107
	dc.l	$01BCDE34,$00D814FA,$01CBDD10,$016AFDBF,$008F8199
	dc.l	$0020F8E9,$00AA8316,$01108B6C,$018FF919,$005E4570
	dc.l	$00B5F6E3,$0010683B,$005553CA,$0141352B,$00DFCCBA
	dc.l	$013B6D3A,$01EC5879,$00727D36,$00A36C2A,$00CBBF87
	dc.l	$01AC1FAC,$0063B30D,$015B008A,$00D5930A,$005DB8BE
	dc.l	$00C2F0DE,$017852B8,$007A21C1,$0108833A,$0093C624
	dc.l	$01E87CFA,$019B7D97,$002C133D,$01912EC0,$015004B7
	dc.l	$0083BDD5,$005DBBBA,$00361419,$0086D5F3,$01A37322
	dc.l	$01552321,$00C94699,$00947FBB,$01F042C1,$002C6C34
	dc.l	$0172A4A6,$00D80BA2,$0159023C,$0123B5A9,$00998666
	dc.l	$00AF444D,$016B550D,$00D2630A,$003AC8FC,$001FE374
	dc.l	$014FD7C1,$004B7834,$00963ACA,$016537D1,$012CEBDA
	dc.l	$00D0AA35,$00D44282,$01E96B69,$0117F62F,$01AD4BEB
	dc.l	$014E1755,$0043919B,$0151D432,$00AEDD1A,$01289E92
	dc.l	$00ED4B9B,$0146246F,$0105C590,$0152DB29,$01254F26

	dc.l	$01C83E9A,$01E5A7A2,$015B31B0,$00F45A46,$006A8BA3	* 10 move puzzles
	dc.l	$01B006AC,$016BEECE,$00F32974,$019F4FC7,$019E6379
	dc.l	$01E122D6,$002F7B8A,$00CBBB06,$012E3AC4,$0132A9F9
	dc.l	$00AF61BB,$01320C06,$01B56FD6,$0070AE90,$002B7835
	dc.l	$009BFD83,$017B2D76,$0139335B,$00EDE726,$00A9EE09
	dc.l	$00FE8672,$00B2C136,$01842FD0,$0196349E,$01668405
	dc.l	$00D09FCE,$01AF86B5,$00DF0C1C,$0098E5C8,$00AED194
	dc.l	$0012314A,$004686E8,$00D6B8ED,$00D724BF,$00B903EB
	dc.l	$00BF79B5,$011A40B4,$008696E3,$00D73C74,$00D63EE8
	dc.l	$006B6BAD,$01855EC1,$006A977B,$004AF513,$00B193B0
	dc.l	$01BC1933,$0137F22F,$01FF0055,$015AC3EB,$005A3D92
	dc.l	$0003CB0E,$01140D0D,$00EF460D,$00DA908A,$01E8A331
	dc.l	$00DF60D5,$001EF495,$0117E13F,$0136BDBC,$0007D535
	dc.l	$012AE96C,$00327EB5,$01BD0CB2,$00DFC8DB,$019A23C0
	dc.l	$00697CE8,$01A1519B,$006FBC75,$0091CE34,$00DA8EF6
	dc.l	$0058D466,$00F4451E,$00449514,$01CA8263,$0195C934
	dc.l	$006633F8,$00A81654,$004DC74D,$01023EA4,$000FC72E
	dc.l	$01D22CAD,$00A5B01D,$01B41537,$012CFEBC,$006B92C5
	dc.l	$00411DFF,$00E90255,$01878E75,$005D84F1,$015E79D9
	dc.l	$01BB5EFA,$01630857,$0090960B,$0003B0E8,$008494D1

	dc.l	$00E5F083,$0145322A,$019ACB45,$00EAF129,$008AE025	* 11 move puzzles
	dc.l	$00966997,$00B50B56,$013C2A0F,$002D79D6,$001FF889
	dc.l	$00F48AA0,$015DF0CD,$00DF4181,$01677E72,$01D92674
	dc.l	$0059EAFF,$01842F97,$0063CB0F,$00FFDA37,$00990794
	dc.l	$0101F5BB,$000FD205,$0185C6F8,$0084B714,$002AECD3
	dc.l	$003E3F77,$0103B73A,$00ECC033,$009A1CF2,$0006EA5D
	dc.l	$01DCB23F,$00D0FF89,$01CFD89B,$00CB8FA1,$015819B4
	dc.l	$001FC82C,$00CAAF60,$0109AE75,$00C8D289,$01F7B795
	dc.l	$01CA41F3,$00F16F8F,$004178F0,$0002C030,$01B86CBF
	dc.l	$003E56C3,$01B41344,$01F9F115,$0106F0AC,$01F25D6C
	dc.l	$01769412,$008B9ABB,$002EDF9F,$01C9ABF5,$005F127E
	dc.l	$0098C03E,$015686DD,$000AD9C0,$0124741D,$00B56230
	dc.l	$00113DC7,$016B4932,$01E38E9E,$0088BA68,$00C4322E
	dc.l	$01568F23,$01D96097,$00DEE619,$00B1DDFB,$01138D5B
	dc.l	$005F1A3B,$008D62A5,$0094C9D5,$01934CA3,$0041E830
	dc.l	$00D5EE92,$019364CD,$0191AAE5,$00CF0FD0,$00F6A937
	dc.l	$005F9150,$01EB7061,$0143C9E2,$01F42672,$00DF7E27
	dc.l	$006EC59A,$0004A213,$0165C122,$00481A68,$00BB0735
	dc.l	$009235F6,$008B8B38,$0187FB45,$004ECAAE,$00D805D3
	dc.l	$00CE1EA5,$00ED0997,$004DD912,$00E82FC5,$00147302

	dc.l	$0114EB45,$01E86656,$009A6DE3,$00A909C8,$01212281	* 12 move puzzles
	dc.l	$00532E72,$00CC98F5,$0163890F,$000EF834,$0131F887
	dc.l	$009874B4,$007822BE,$01B9EEFF,$01564956,$0131B939
	dc.l	$010A9A2A,$01B86C31,$0120CF06,$00660620,$000D579F
	dc.l	$01D05C55,$0163D54D,$00B0D71A,$011461F6,$0109FC50
	dc.l	$0135FAE7,$0150CA67,$0124C807,$00E5AEEC,$00A765AC
	dc.l	$01D3E455,$01EDE225,$00C4B97D,$0138E228,$001F1455
	dc.l	$000E968B,$00AA51B4,$005D034F,$01708E10,$00AFEB0F
	dc.l	$016C4E45,$0184B2C8,$00262534,$01ED9F22,$00986175
	dc.l	$0170D42C,$016C150C,$00F6F9D2,$00F2DE60,$012CB1F0
	dc.l	$000F49F6,$009BE0CE,$017024DE,$0175B6EB,$01017A9F
	dc.l	$01E9D16E,$0087CCFB,$00B07ABC,$01513E90,$005A115E
	dc.l	$01F16E75,$0195DB2C,$018CAC06,$00CEEFE6,$009A5F66
	dc.l	$00C48E33,$01710266,$01332091,$01F55A7F,$01A09350
	dc.l	$015E5261,$00D12E68,$01F4D595,$0162C811,$01054CA9
	dc.l	$007C370D,$0024690A,$01A68C17,$0090538C,$00A42CAF
	dc.l	$01175707,$01C19D15,$01516340,$001E1BC3,$0091E82C
	dc.l	$005B2D0A,$0034A649,$011234D1,$0120D2AC,$001EDC29
	dc.l	$011F8EA7,$00697D05,$018EB92E,$00CE1778,$010D88F2
	dc.l	$0116C36F,$019028E8,$01021824,$005AAAEF,$0090E9B4

	dc.l	$01BDB2EB,$01CD2B56,$00DBA9CD,$007A300F,$002F3F96	* 13 move puzzles
	dc.l	$0026775B,$001C3B46,$00B29CF0,$010BEB4E,$0083CDDF
	dc.l	$01BAF642,$01C04851,$01922191,$014EC6E5,$01131B19
	dc.l	$00B0A56C,$00D1495B,$0116CB8D,$000D7417,$019EFD0C
	dc.l	$01D970CB,$002A1181,$01E31EE8,$0120904A,$01F72C70
	dc.l	$00F28E82,$00924492,$012174E6,$01326B4D,$01C8F5FE
	dc.l	$01E9EBC0,$01BEA5F5,$00E0F8DA,$00DD6893,$01E9C0AA
	dc.l	$00A82656,$00923181,$01696C76,$0099A903,$01BA55EA
	dc.l	$01565DF3,$013B426D,$00963956,$0033D046,$0107F84E
	dc.l	$017C4A01,$00B3E0BD,$010BB99A,$003FCB4E,$00AA2429
	dc.l	$01031892,$0005BE45,$0150F138,$00617270,$01583D8C
	dc.l	$0053EB60,$01D14517,$01F840CB,$001D4A1D,$007999A2
	dc.l	$016C85B9,$01E443CA,$0145C52F,$0145A0CD,$00E5A7F8
	dc.l	$019A5AC5,$00D7BA52,$01FAFC35,$00AA2C45,$0144CC8D
	dc.l	$01567AAB,$00596496,$016130B4,$01703055,$004CD1F0
	dc.l	$01BB7BC5,$01B1254A,$00D7980F,$0120C612,$00CE4C43
	dc.l	$005978D1,$011201AE,$00C81532,$01440D05,$01AE9B7B
	dc.l	$016F301E,$01333C76,$00AA1A28,$007AC5D1,$004A69BA
	dc.l	$00C85138,$00A899E8,$0122BC17,$002879CD,$0028D032
	dc.l	$0144AB54,$01F7E66C,$01909491,$01987BA7,$018FFDBB

	dc.l	$004A7012,$00A326A7,$00D4C193,$008E5F60,$01080C7D	* 14 move puzzles
	dc.l	$0004A1C8,$018C8A81,$001E8D37,$0168E00A,$0014581D
	dc.l	$00A18BE8,$01FAF96B,$00AD1FE7,$01034BC5,$01028943
	dc.l	$0166E320,$01CD23B4,$0042D9B8,$019DEA9C,$005BA25C
	dc.l	$0148C151,$01B9DCB3,$00937FEE,$00F2B559,$0107B5B7
	dc.l	$017F9F4B,$00927331,$010BCC32,$01C45DF4,$004880F5
	dc.l	$0029A44F,$01B12B7B,$0116AAC0,$0128218B,$00178903
	dc.l	$007E0722,$0100B072,$01748A15,$00394D70,$01544AD8
	dc.l	$00C4EBB2,$015BDE1D,$00058EBC,$017E840A,$0120F4D0
	dc.l	$01A96126,$014E6851,$0154C06B,$00908189,$008109E0
	dc.l	$00350CA4,$01AD1DDD,$01438DB3,$00BD1A43,$0155CC7D
	dc.l	$01901911,$019C8E7E,$00C69FC1,$0051438A,$00A746AA
	dc.l	$015F1F78,$004FC9D2,$006AD881,$018123D0,$00C80492
	dc.l	$004AAAB5,$01667A19,$00D579F9,$01C97999,$01943C56
	dc.l	$0165FE5C,$000F6673,$0079B28F,$00B2E558,$002F6A3C
	dc.l	$0155500B,$00D10C51,$00A56ED1,$0118702D,$00C53450
	dc.l	$01466707,$00B30963,$0131A031,$00541956,$00266EF3
	dc.l	$0071D193,$0113937B,$000B39A0,$01A04155,$0102D9AC
	dc.l	$00A29913,$0188F0AE,$01E244CF,$0132FCE8,$0105D9DD
	dc.l	$011E000A,$01EB2E71,$00D3F51E,$0159F329,$00836FB9

	dc.l	$01C3B334,$00A57F71,$00DD8D11,$00088151,$0040E138	* 15 move puzzles
	dc.l	$00EEE089,$0170FF57,$00D56331,$00D6018D,$007E0FC0
	dc.l	$012584D7,$01341113,$0010E25D,$0040E063,$000F5D53
	dc.l	$01B8ED4A,$01DA44B7,$01150794,$0052DD56,$004FC130
	dc.l	$0143D785,$00089140,$01140D56,$01149240,$01956045
	dc.l	$00517013,$007F13C7,$00909029,$01340059,$0076E1E0
	dc.l	$01730572,$01421651,$001EC897,$01521095,$015F93F5
	dc.l	$00154550,$01440D53,$01424485,$01B8FC00,$00A8FD4A
	dc.l	$00E27EB5,$01CA4048,$008CC1FC,$01B56C11,$0042406B
	dc.l	$01577DD5,$01151231,$01189231,$01590244,$001907E4
	dc.l	$0197C04D,$0059C953,$0008901B,$01AC006B,$007290A7
	dc.l	$01220FB5,$018C8263,$01548124,$01617D0D,$0174DCC4
	dc.l	$01CB9759,$01113399,$00728459,$01DBC029,$01317C53
	dc.l	$01C2FCBC,$01BD937B,$00489235,$011504F4,$01C7C328
	dc.l	$0170EE1D,$00810102,$01811112,$01B1831B,$01CA129C
	dc.l	$015F91CE,$00D42CB9,$016DE10A,$00B4ADC4,$0185EC32
	dc.l	$00C79449,$00524494,$0185FD78,$00448135,$017357A6
	dc.l	$00240048,$01C8E11B,$00908163,$014CB0FC,$011CE14D
	dc.l	$007E83A7,$01B0101B,$01617F36,$00D5610A,$01190131
	dc.l	$00050231,$011DFD4A,$0004831B,$009D419D,$01651E60


*************************************************************************************
*
* variables used

variables						* points to variables base

	OFFSET	0				* going to use relative addressing

help_avail
	ds.l	1					* available help bit pattern
moves_made
	ds.l	1					* moves so far bit pattern
game_board
	ds.l	1					* game board bit pattern
last_board
	ds.l	1					* game board bit pattern for the last game

draw_board
	ds.w	1					* draw the board flag
display_board
	ds.l	1					* display bit pattern
game_mode
	ds.w	1					* game mode
last_mode
	ds.w	1					* last game mode
select_mode
	ds.w	1					* select mode

play_level
	ds.w	1					* play level
play_sound
	ds.w	1					* play sound flag

chase_seq
	ds.w	1					* chase count
chase_loop
	ds.w	1					* chase loop count
button_count
	ds.w	1					* button press count

button_l
	ds.w	1					* left mouse button flag
button_r
	ds.w	1					* right mouse button flag
mousex
	ds.w	1					* mouse x co-ordinate
mousey
	ds.w	1					* mouse y co-ordinate
PRNlword
	ds.l	1					* pseudo random longword

help_count
	ds.w	1					* help count remaining
solution
	ds.l	2					* help solution longword pair

file_id
	ds.l	1					* file ID longword

im_x
	ds.w	1					* image x origin
im_y
	ds.w	1					* image y origin

im_width
	ds.l	1					* image width
im_height
	ds.l	1					* image height
im_depth
	ds.w	1					* image depth in bits per pixel
im_palette
	ds.l	256					* image palette space if needed

im_file
							* the rest of the memory is the file buffer

	END	start


*************************************************************************************